/*
 * sounddummy.c - Implementation of the dummy sound device
 *
 * Written by
 *  Teemu Rantanen <tvr@cs.hut.fi>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

//#include "vice.h"

#include <stdio.h>

#include <SLES/OpenSLES.h>
#include <SLES/OpenSLES_Android.h>
#include <log.h>

#include "sound.h"

static SLObjectItf engineObject;
static SLEngineItf engineEngine;
static SLObjectItf outputMixObject;
static SLuint32 speakers;
static SLuint32 sr;
static SLObjectItf bqPlayerObject;
static SLPlayItf bqPlayerPlay;
static SLBufferQueueItf bqPlayerBufferQueue;

static int num_channels;

static void bqPlayerCallback(SLAndroidSimpleBufferQueueItf bq, void *context);

//static int bufferspace=1024;
//static int fs;

static int opensles_init(const char *param, int *speed, int *fragsize, int *fragnr, int *channels) {
    //bufferspace=(*fragsize)*4;
    //fs=fragsize;

    //return 0;
    log_message(LOG_DEFAULT, "opensles_init speed: %d, fragsize: %d: fragnr: %d, channels: %d", *speed, *fragsize, *fragnr, *channels);
    //param="default";

    SLresult result;
    result=slCreateEngine(&engineObject, 0, NULL, 0, NULL, NULL);
    result=(*engineObject)->Realize(engineObject, SL_BOOLEAN_FALSE);
    result=(*engineObject)->GetInterface(engineObject,
                                     SL_IID_ENGINE, &engineEngine);
    const SLInterfaceID ids[] = {SL_IID_VOLUME};
    const SLboolean req[] = {SL_BOOLEAN_FALSE};
    result=(*engineEngine)->CreateOutputMix(engineEngine, &outputMixObject, 1, ids, req);
    result=(*outputMixObject)->Realize(outputMixObject, SL_BOOLEAN_FALSE);
    SLDataLocator_AndroidSimpleBufferQueue loc_bufq =
                           {SL_DATALOCATOR_ANDROIDSIMPLEBUFFERQUEUE, 2};

    switch(*speed) {
        case 8000:
            sr = SL_SAMPLINGRATE_8;
            break;
        case 11025:
            sr = SL_SAMPLINGRATE_11_025;
            break;
        case 16000:
            sr = SL_SAMPLINGRATE_16;
            break;
        case 22050:
            sr = SL_SAMPLINGRATE_22_05;
            break;
        case 24000:
            sr = SL_SAMPLINGRATE_24;
            break;
        case 32000:
            sr = SL_SAMPLINGRATE_32;
            break;
        case 44100:
            sr = SL_SAMPLINGRATE_44_1;
            break;
        case 48000:
            sr = SL_SAMPLINGRATE_48;
            break;
        case 64000:
            sr = SL_SAMPLINGRATE_64;
            break;
        case 88200:
            sr = SL_SAMPLINGRATE_88_2;
            break;
        case 96000:
            sr = SL_SAMPLINGRATE_96;
            break;
        case 192000:
            sr = SL_SAMPLINGRATE_192;
            break;
        default:
            sr = SL_SAMPLINGRATE_44_1;
    }

    num_channels=*channels;

    if(num_channels>1) speakers=SL_SPEAKER_FRONT_LEFT | SL_SPEAKER_FRONT_RIGHT;
    else speakers = SL_SPEAKER_FRONT_CENTER;

    SLDataFormat_PCM format_pcm = {SL_DATAFORMAT_PCM, num_channels, sr,
               SL_PCMSAMPLEFORMAT_FIXED_16, SL_PCMSAMPLEFORMAT_FIXED_16,
               speakers, SL_BYTEORDER_LITTLEENDIAN};
    SLDataSource audioSrc = {&loc_bufq, &format_pcm};

    SLDataLocator_OutputMix loc_outmix = {SL_DATALOCATOR_OUTPUTMIX,
                                                outputMixObject};
    SLDataSink audioSnk = {&loc_outmix, NULL};

    const SLInterfaceID ids1[] = {SL_IID_ANDROIDSIMPLEBUFFERQUEUE};
    const SLboolean req1[] = {SL_BOOLEAN_TRUE};
    result = (*engineEngine)->CreateAudioPlayer(engineEngine,
                    &bqPlayerObject, &audioSrc, &audioSnk,
                     1, ids1, req1);

    result = (*bqPlayerObject)->Realize(bqPlayerObject, SL_BOOLEAN_FALSE);
    result = (*bqPlayerObject)->GetInterface(bqPlayerObject,
                                 SL_IID_PLAY,&bqPlayerPlay);

    result = (*bqPlayerObject)->GetInterface(bqPlayerObject,
       SL_IID_ANDROIDSIMPLEBUFFERQUEUE, &(bqPlayerBufferQueue));
    result = (*bqPlayerBufferQueue)->RegisterCallback(
                      bqPlayerBufferQueue, (void *)bqPlayerCallback, NULL);
    result = (*bqPlayerPlay)->SetPlayState(bqPlayerPlay,
                                            SL_PLAYSTATE_PLAYING);

    return 0;
}

static int opensles_write(SWORD *pbuf, size_t nr) {
    //log_message(LOG_DEFAULT, "opensles_write %d", (int)nr);
    //bufferspace=(int)nr+64;
    //return 0;
    //SLuint32 size=(SLuint32)nr/num_channels;
    //bufferspace=(int)nr*2;
    SLuint32 size=(SLuint32)nr;
    //bufferspace-=256;
    SLresult result;
    result=(*bqPlayerBufferQueue)->Enqueue(bqPlayerBufferQueue, (const void *)pbuf, size*2);
    return 0;
}

static void bqPlayerCallback(SLAndroidSimpleBufferQueueItf bq, void *context) {
    //log_message(LOG_DEFAULT, "opensles callback");
}

//static int opensles_bufferspace(void) {
    //log_message(LOG_DEFAULT, "opensles bufferspace");
    //return 1024;
    //return bufferspace;
    //return 8192;
    //return 896;
//}

static void opensles_close() {
    //log_message(LOG_DEFAULT, "opensles close");
}

static int opensles_suspend() {
    //log_message(LOG_DEFAULT, "opensles suspend");
    return 0;
}

static int opensles_resume() {
    //log_message(LOG_DEFAULT, "opensles resume");
    return 0;
}

static sound_device_t sound_opensles_device = {
                "dummy",
                opensles_init,
                opensles_write,
                NULL,
                NULL,
                0, //opensles_bufferspace,
                opensles_close,
                opensles_suspend,
                NULL, //opensles_resume,
                1,
                2
        };

int sound_init_dummy_device(void) {
    //log_message(LOG_DEFAULT, "sound_opensles register");
    return sound_register_device(&sound_opensles_device);
}
