/*
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software{} you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation{} either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY{} without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program{} if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */


//#include "types.h"
#include <malloc.h>
#include <color.h>
#include <palette.h>
#include <unistd.h>
#include <vsyncapi.h>
#include "video.h"
#include "videoarch.h"
#include "log.h"

/* video filter type, resource "CHIPFilter" */
#define VIDEO_FILTER_NONE         0
#define VIDEO_FILTER_CRT          1
#define VIDEO_FILTER_SCALE2X      2

/* These constants are used to configure the video output.  */

/* no video output (dummy) */
//#define VIDEO_RENDER_NULL       0
/* PAL or NTSC TV/Monitor (like VIC, VIC-II or TED) */
//#define VIDEO_RENDER_PAL_1X1    1
//#define VIDEO_RENDER_PAL_2X2    2
/* no filtering */
//#define VIDEO_RENDER_RGB_1X1    3
//#define VIDEO_RENDER_RGB_1X2    4 /* needed for y-stretch */
//#define VIDEO_RENDER_RGB_2X2    5
//#define VIDEO_RENDER_RGB_2X4    6 /* needed for y-stretch */
/* RGB(I) or monochrome Monitor (CRTC and VDC) */
//#define VIDEO_RENDER_CRT_1X1    7 /* FIXME: to be written */
//#define VIDEO_RENDER_CRT_1X2    8 /* needed for y-stretch */
//#define VIDEO_RENDER_CRT_2X2    9
//#define VIDEO_RENDER_CRT_2X4   10 /* needed for y-stretch */


uint32_t *video_buffer;
unsigned int video_width;
unsigned int video_height;
unsigned int src_video_width;
unsigned int video_x_adjust;
unsigned int video_y_adjust;
volatile int video_buf_updated;

//volatile int vice_paused;

int video_arch_cmdline_options_init(void) {
    return 0;
}

int video_init(void) {
    return 0;
}

void video_shutdown(void) {

}

struct video_canvas_s *video_canvas_create(struct video_canvas_s *canvas,
                                                  unsigned int *width, unsigned int *height,
                                                  int mapped) {
    //log_message(LOG_DEFAULT, "video_canvas_create: canvas: %x, width: %d, height: %d mapped: %d", (int)canvas, *width, *height, mapped );
    //log_message(LOG_DEFAULT, "video_canvas_create: canvas: %x, draw_buffer-width: %d, draw_buffer-height: %d",
    //            (int)canvas, canvas->draw_buffer->draw_buffer_width, canvas->draw_buffer->draw_buffer_height);

    //canvas->videoconfig->rendermode=VIDEO_RENDER_PAL_1X1;

    //video_buffer[6400]=0xf0f0;
    //video_buffer=(uint32_t *)malloc(384*272*4);
    //video_canvas_set_palette(canvas, canvas->palette);
    //canvas->videoconfig->rendermode=3;
    canvas->draw_buffer->canvas_physical_width=video_width;
    canvas->draw_buffer->canvas_physical_height=video_height;
    canvas->initialized=1;
    //canvas->created=1;
    video_color_update_palette(canvas);
    return canvas;
}

void video_arch_canvas_init(struct video_canvas_s *canvas) {
    canvas->video_draw_buffer_callback=NULL;
}

void video_canvas_refresh(struct video_canvas_s *canvas,
                                 unsigned int xs, unsigned int ys,
                                 unsigned int xi, unsigned int yi,
                                 unsigned int w, unsigned int h) {

    //while(vice_paused) sleep(1);

    //if(w>0)
    //log_message(LOG_DEFAULT, "video_canvas_refresh: canvas: %x, xs %d, ys %d xi %d, yi %d, w %d, h %d", (int)canvas, xs, ys, xi, yi, w, h);
    //log_message(LOG_DEFAULT, "dbuffer_w: %d, dbuffer_h: %d, canvas_w: %d, canvas_h: %d",
    //            canvas->draw_buffer->draw_buffer_width,
    //            canvas->draw_buffer->draw_buffer_height,
    //            canvas->draw_buffer->canvas_width,
    //            canvas->draw_buffer->canvas_height
    //    );
    if(canvas==NULL||canvas->palette==NULL) return;
    if(canvas->initialized==0) return;

    //log_message(LOG_DEFAULT, "video_canvas_refresh: canvas: %x, xs %d, ys %d xi %d, yi %d, w %d, h %d", (int)canvas, xs, ys, xi, yi, w, h);
    // 07-08 21:09:57.529 4421-4506/org.gp2x.cvice D/archdep_default_logger: : video_canvas_refresh: canvas: a4926f70, xs 0, ys 16 xi 0, yi 0, w 384, h 272

    //if(w>384) w=384;
    //if(h>272) h=272;
    //w=384;
    //h=272;
    //w=504;
    //int src_video_width=504;

    //#define DEPTH 24
    // bytes per line
    //#define PITCH 384*2*4
    //video_canvas_render(canvas, (BYTE *)video_buffer, w, h, 0, 0, xi, yi, PITCH, DEPTH);
    //log_message(LOG_DEFAULT, "canvas x: %d", canvas->draw_buffer->draw_buffer[6400]);
    //log_message(LOG_DEFAULT, "rendermode: %d", canvas->videoconfig->rendermode);
    unsigned int x, y;
    unsigned int colour, red, green, blue;
    for(y=0; y<video_height; y++) {
        for(x=0; x<video_width; x++) {
            colour=canvas->draw_buffer->draw_buffer[((y+video_y_adjust)*src_video_width)+x+video_x_adjust];
            red=(uint8_t)canvas->palette->entries[colour].red;
            green=(uint8_t)canvas->palette->entries[colour].green;
            blue=(uint8_t)canvas->palette->entries[colour].blue;
            video_buffer[(y*video_width)+x]=(0xff<<24)|(blue<<16)|(green<<8)|red;
        }
    }

    video_buf_updated=1;

    /*static unsigned long start=0;
    static int frames;
    if(start==0) start=vsyncarch_gettime();
    unsigned long now=vsyncarch_gettime();
    frames++;
    unsigned long elapsed=now-start; elapsed++;
    int fps=(int)frames/elapsed;
    log_message(LOG_DEFAULT, "frames: %d, fps: %d", frames, fps);*/

}

int video_canvas_set_palette(struct video_canvas_s *canvas,
                                    struct palette_s *palette)
{
    //log_message(LOG_DEFAULT, "video_canvas_set_palette: %x", palette);
    canvas->palette=palette;
    //video_color_update_palette(canvas);
    //color_init();
    //color_alloc_colors(canvas, palette, NULL);

    return 0;
}

void video_canvas_destroy(struct video_canvas_s *canvas)
{
    canvas->initialized=0;
    //free(video_buffer);
    //video_buffer=0;
}

//void video_canvas_map(struct video_canvas_s *canvas)
//{
//}

//void video_canvas_unmap(struct video_canvas_s *canvas)
//{
//}

void video_canvas_resize(struct video_canvas_s *canvas, char resize_canvas) {

}

char video_canvas_can_resize(struct video_canvas_s *canvas) {
    return 0;
}

int video_arch_resources_init(void) {
    return 0;
}

void video_arch_resources_shutdown(void) {

}

void fullscreen_capability(cap_fullscreen_t *cap_fullscreen) {

}
