/*
 * vsyncapi.h - general vsync archdependant functions
 *
 * Written by
 *  Dag Lem <resid@nimrod.no>
 *  Thomas Bretz <tbretz@gsi.de>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software{} you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation{} either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY{} without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program{} if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include <sys/time.h>
#include <unistd.h>
#include <time.h>

#include <autostart.h>
#include <machine.h>
#include <kbdbuf.h>
#include <stdlib.h>

#include "log.h"
#include "joy.h"
#include "kbd.h"

volatile int vice_uireset;
volatile int vice_paused;
volatile int vice_exit;
volatile int menu_load_file;
const char *menu_selected_file;

struct video_canvas_s;

typedef void (*void_hook_t)(void);

/* number of timer units per second - used to calc speed and fps */
signed long vsyncarch_frequency(void) {
    //log_message(LOG_DEFAULT, "vsyncarch_frequency");
    /* Microseconds resolution. */
    return 1000000;
}

unsigned long start_time;

/* provide the actual time in timer units */
unsigned long vsyncarch_gettime(void) {
    struct timeval now;
    gettimeofday(&now, NULL);
    //unsigned short x;
    //log_message(LOG_DEFAULT, "vsyncarch_gettime %lu %lu %d", 1000000UL*now.tv_sec+now.tv_usec, ULONG_MAX, sizeof(x));

    //return 1000000UL * now.tv_sec + now.tv_usec;
    //return (1000000UL*(now.tv_sec%100))+now.tv_usec;

    //struct timespec now;
    //clock_gettime(CLOCK_MONOTONIC, &now);
    //return (int64_t) now.tv_sec*1000000000LL + now.tv_nsec;

    if(start_time==0) start_time=(unsigned long)now.tv_sec;
    unsigned long elapsed_time=now.tv_sec-start_time;
    //log_message(LOG_DEFAULT, "vsyncarch_gettime %lu %lu %d", 1000000UL*elapsed_time+now.tv_usec, ULONG_MAX, sizeof(x));

    return 1000000UL*elapsed_time+now.tv_usec;
}

/* call when vsync_init is called */
void vsyncarch_init(void) {

}

/* display speed(%) and framerate(fps) */
void vsyncarch_display_speed(double speed, double fps, int warp_enabled) {
    //char string[255];
    //snprintf(string, 255, "speed: %f, fps: %f", speed, fps);
    //log_message(LOG_DEFAULT, "%s", string);
}

/* sleep the given amount of timer units */
void vsyncarch_sleep(signed long delay) {
    //log_message(LOG_DEFAULT, "vsyncarch_sleep %ld", delay);
    usleep(delay);
}

//#if defined (HAVE_OPENGL_SYNC) && !defined(USE_SDLUI) && !defined(USE_SDLUI2)
/* synchronize with vertical blanks */
//void vsyncarch_verticalblank(struct video_canvas_s *c, float rate, int frames) {
//}

/* keep vertical blank sync prepared */
//void vsyncarch_prepare_vbl(void) {
//}
//#endif

/* this is called before vsync_do_vsync does the synchroniation */
void vsyncarch_presync(void) {
    //log_message(LOG_DEFAULT, "vsyncarch_presync");
    kbdbuf_flush();
}

/* this is called after vsync_do_vsync did the synchroniation */
void vsyncarch_postsync(void) {
    //log_message(LOG_DEFAULT, "vsyncarch_postsync");
    if(vice_uireset) {
        vice_uireset=0;
        machine_trigger_reset(MACHINE_RESET_MODE_HARD);
    }
    if(menu_load_file) {
        menu_load_file=0;
        autostart_autodetect(menu_selected_file, NULL, 0, 0);
    }

    while(vice_paused) {
        sleep(1);
        if(vice_exit) exit(0);
    }

    check_keypress();
    joystick_update();
}
