/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

package org.gp2x.cvice;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.opengl.GLUtils;
import android.os.Bundle;
import android.content.Context;
import android.Manifest.permission;
import android.opengl.GLES20;
import android.opengl.Matrix;
import android.os.Vibrator;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.view.KeyEventCompat;
import android.util.Log;
import android.view.KeyEvent;

import com.google.vr.sdk.base.GvrActivity;
import com.google.vr.sdk.base.GvrView;
//import com.google.vr.sdk.audio.GvrAudioEngine;
import com.google.vr.sdk.base.AndroidCompat;
import com.google.vr.sdk.base.Eye;
import com.google.vr.sdk.base.HeadTransform;
import com.google.vr.sdk.base.Viewport;

import javax.microedition.khronos.egl.EGLConfig;

public class MainActivity extends GvrActivity implements GvrView.StereoRenderer {

    private static final String TAG = "VRVice";

    VICEMachine vice_machine;
    private ByteBuffer machine_video_bbuf;
    private byte[] machine_video_buf;
    int machine_video_width;
    int machine_video_height;

    UserInterface ui;

    protected float[] modelScreen;
    protected float[] screenPosition;
    protected float[] screenRotation;
    protected float[] modelMenu;
    protected float[] menuPosition;
    protected float[] menuRotation;

    private static final float Z_NEAR = 0.1f;
    private static final float Z_FAR = 100.0f;

    private static final float CAMERA_Z = 0.01f;
    //private static final float TIME_DELTA = 0.3f;

    //private static final float YAW_LIMIT = 0.12f;
    //private static final float PITCH_LIMIT = 0.12f;

    private static final int COORDS_PER_VERTEX = 3;

    // We keep the light always position just above the user.
    //private static final float[] LIGHT_POS_IN_WORLD_SPACE = new float[]{0.0f, 2.0f, 0.0f, 1.0f};

    // Convenience vector for extracting the position from a matrix via multiplication.
    private static final float[] POS_MATRIX_MULTIPLY_VEC = {0, 0, 0, 1.0f};

    //private static final float MIN_MODEL_DISTANCE = 3.0f;
    //private static final float MAX_MODEL_DISTANCE = 7.0f;

    private final float[] lightPosInEyeSpace = new float[4];

    private int screenProgram;
    private FloatBuffer screenTexVertices;
    private FloatBuffer screenVertices;
    private int screenTexCoordinate;
    private int screenPositionParam;
    private int screenModelViewProjectionParam;

    private int TextureHandle[];

    private int menuProgram;
    private FloatBuffer menuTexVertices;
    private FloatBuffer menuVertices;
    private int menuTexCoordinate;
    private int menuPositionParam;
    private int menuModelViewProjectionParam;

    private float[] camera;
    private float[] view;
    private float[] headView;
    private float[] forwardView;
    private float[] modelViewProjection;
    private float[] modelView;

    private float[] tempPosition;
    private float[] headRotation;

    //private float objectDistance = MAX_MODEL_DISTANCE / 2.0f;
    //private float floorDepth = 20f;

    private Vibrator vibrator;

    //private GvrAudioEngine gvrAudioEngine;
    //private volatile int sourceId = GvrAudioEngine.INVALID_ID;
    //private volatile int successSourceId = GvrAudioEngine.INVALID_ID;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        get_android_permissions();

        String machine="x64";
        try {
            ActivityInfo ai = getPackageManager().getActivityInfo(getComponentName(), PackageManager.GET_ACTIVITIES|PackageManager.GET_META_DATA);
            machine=ai.metaData.getString("machine");
            Log.i(TAG, "machine name: "+machine);
        } catch (Exception e) {
            e.printStackTrace();
        }

        //machine="xplus4";
        switch(machine) {
            default:
            case "x64":
            //if (machine.equals("x64")) {
                machine_video_width = 384; //504;
                machine_video_height = 272;
                machine_video_bbuf = ByteBuffer.allocateDirect(machine_video_width * machine_video_height * 4);
                machine_video_buf = machine_video_bbuf.array();
                vice_machine = new VICEMachine(VICEMachine.vice_machine.X64, this.getApplicationContext(), machine_video_buf);
                break;
            case "x64sc":
            //} else if (machine.equals("x64sc")) {
                machine_video_width = 384;
                machine_video_height = 272;
                machine_video_bbuf = ByteBuffer.allocateDirect(machine_video_width * machine_video_height * 4);
                machine_video_buf = machine_video_bbuf.array();
                vice_machine = new VICEMachine(VICEMachine.vice_machine.X64SC, this.getApplicationContext(), machine_video_buf);
                break;
            case "xplus4":
            //} else if (machine.equals("xplus4")) {
                machine_video_width = 384;
                machine_video_height = 312;
                machine_video_bbuf = ByteBuffer.allocateDirect(machine_video_width * machine_video_height * 4);
                machine_video_buf = machine_video_bbuf.array();
                vice_machine = new VICEMachine(VICEMachine.vice_machine.XPLUS4, this.getApplicationContext(), machine_video_buf);
                break;
            case "xvic":
            //} else if (machine.equals("xvic")) {
                machine_video_width = 480;
                machine_video_height = 312;
                machine_video_bbuf = ByteBuffer.allocateDirect(machine_video_width * machine_video_height * 4);
                machine_video_buf = machine_video_bbuf.array();
                vice_machine = new VICEMachine(VICEMachine.vice_machine.XVIC, this.getApplicationContext(), machine_video_buf);
                break;
            //}
        }
        vice_machine.start();

        ui=new UserInterface(vice_machine);

        initializeGvrView();

        modelScreen = new float[16];
        modelMenu = new float[16];
        camera = new float[16];
        view = new float[16];
        modelViewProjection = new float[16];
        modelView = new float[16];
        //modelFloor = new float[16];
        tempPosition = new float[4];
        // Model first appears directly in front of user.
        //modelPosition = new float[] {0.0f, 0.0f, -MAX_MODEL_DISTANCE / 2.0f};
        screenPosition = new float[] {0.0f, 0.0f, -40.0f};
        screenRotation = new float[] {0.0f, 0.0f, 0.0f};
        menuPosition = new float[] {-2.3f, -1.8f, -3.5f};
        menuRotation = new float[] {-10.0f, 30.0f, 0.0f};
        headRotation = new float[4];
        headView = new float[16];
        forwardView = new float[3];
        vibrator = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);

        // Initialize 3D audio engine.
        //gvrAudioEngine = new GvrAudioEngine(this, GvrAudioEngine.RenderingMode.BINAURAL_HIGH_QUALITY);
    }

    public void initializeGvrView() {
        setContentView(R.layout.activity_main);

        GvrView gvrView = (GvrView) findViewById(R.id.gvr_view);
        gvrView.setEGLConfigChooser(8, 8, 8, 8, 16, 8);

        gvrView.setRenderer(this);
        gvrView.setTransitionViewEnabled(true);

        // Enable Cardboard-trigger feedback with Daydream headsets. This is a simple way of supporting
        // Daydream controller input for basic interactions using the existing Cardboard trigger API.
        gvrView.enableCardboardTriggerEmulation();

        if (gvrView.setAsyncReprojectionEnabled(true)) {
            // Async reprojection decouples the app framerate from the display framerate,
            // allowing immersive interaction even at the throttled clockrates set by
            // sustained performance mode.
            AndroidCompat.setSustainedPerformanceMode(this, true);
        }

        setGvrView(gvrView);
    }

    @Override
    public void onResume() {
        Log.i(TAG, "lifecycle: onResume");
        super.onResume();
        //gvrAudioEngine.resume();
        vice_machine.viceResume();
    }

    @Override
    public void onPause() {
        Log.i(TAG, "lifecycle: onPause");
        //gvrAudioEngine.pause();
        super.onPause();
        vice_machine.vicePause();
    }

    @Override
    public void onStop() {
        Log.i(TAG, "lifecycle: onStop");
        super.onStop();
    }

    @Override
    public void onRestart() {
        Log.i(TAG, "lifecycle: onRestart");
        super.onStop();
    }

    @Override
    public void onDestroy() {
        Log.i(TAG, "lifecycle: onDestroy");
        vice_machine.viceExit();
        super.onDestroy();
    }

    @Override
    public void onRendererShutdown() {
        Log.i(TAG, "onRendererShutdown");
    }

    @Override
    public void onSurfaceChanged(int width, int height) {
        Log.i(TAG, "onSurfaceChanged");
    }

    /**
     * Checks if we've had an error inside of OpenGL ES, and if so what that error is.
     *
     * @param label Label to report in case of error.
     */
    private static void checkGLError(String label) {
        int error;
        while ((error = GLES20.glGetError()) != GLES20.GL_NO_ERROR) {
            Log.e(TAG, label + ": glError " + error);
            throw new RuntimeException(label + ": glError " + error);
        }
    }

    /**
     * Creates the buffers we use to store information about the 3D world.
     * <p>
     * <p>OpenGL doesn't use Java arrays, but rather needs data in a format it can understand.
     * Hence we use ByteBuffers.
     *
     * @param config The EGL configuration used when creating the surface.
     */
    @Override
    public void onSurfaceCreated(EGLConfig config) {
        Log.i(TAG, "onSurfaceCreated");
        //GLES20.glClearColor(7.1f, 7.1f, 7.1f, 0.5f); // Dark background so text shows up well.

        int VertexShader=loadGLShader(GLES20.GL_VERTEX_SHADER, R.raw.texture_vertex);
        int FragmentShader=loadGLShader(GLES20.GL_FRAGMENT_SHADER, R.raw.texture_fragment);

        screenProgram=GLES20.glCreateProgram();
        GLES20.glAttachShader(screenProgram, VertexShader);
        GLES20.glAttachShader(screenProgram, FragmentShader);
        GLES20.glLinkProgram(screenProgram);
        GLES20.glUseProgram(screenProgram);
        checkGLError("Screen program");

        TextureHandle=new int[2];
        GLES20.glGenTextures(1, TextureHandle, 0);
        if (TextureHandle[0] == 0) throw new RuntimeException("Error loading texture.");
        //GLES20.glActiveTexture(GLES20.GL_TEXTURE0);

        GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, TextureHandle[0]);
        GLES20.glTexImage2D(GLES20.GL_TEXTURE_2D, 0, GLES20.GL_RGBA, machine_video_width, machine_video_height, 0, GLES20.GL_RGBA, GLES20.GL_UNSIGNED_BYTE, machine_video_bbuf);
        GLES20.glTexParameteri ( GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_LINEAR );
        GLES20.glTexParameteri ( GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, GLES20.GL_LINEAR );
        GLES20.glTexParameteri ( GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S, GLES20.GL_CLAMP_TO_EDGE );
        GLES20.glTexParameteri ( GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T, GLES20.GL_CLAMP_TO_EDGE );

        //c64BufPositionParam=GLES20.glGetUniformLocation(screenProgram, "u_texture");
        screenTexCoordinate = GLES20.glGetAttribLocation(screenProgram, "a_texCoordinate");
        screenPositionParam = GLES20.glGetAttribLocation(screenProgram, "a_position");
        screenModelViewProjectionParam = GLES20.glGetUniformLocation(screenProgram, "u_sMVP");

        checkGLError("Screen program params");

        ByteBuffer bbScreenTexVertices=ByteBuffer.allocateDirect(WorldLayoutData.TEX_COORDS.length*4);
        bbScreenTexVertices.order(ByteOrder.nativeOrder());
        screenTexVertices=bbScreenTexVertices.asFloatBuffer();
        screenTexVertices.put(WorldLayoutData.TEX_COORDS);
        screenTexVertices.position(0);

        ByteBuffer bbScreenVertices=ByteBuffer.allocateDirect(WorldLayoutData.SCREEN_COORDS.length*4);
        bbScreenVertices.order(ByteOrder.nativeOrder());
        screenVertices=bbScreenVertices.asFloatBuffer();
        screenVertices.put(WorldLayoutData.SCREEN_COORDS);
        screenVertices.position(0);

        menuProgram=GLES20.glCreateProgram();
        GLES20.glAttachShader(menuProgram, VertexShader);
        GLES20.glAttachShader(menuProgram, FragmentShader);
        GLES20.glLinkProgram(menuProgram);
        GLES20.glUseProgram(menuProgram);
        checkGLError("Menu program");

        //menuTextureHandle=new int[1];
        GLES20.glGenTextures(1, TextureHandle, 1);
        if (TextureHandle[1] == 0) throw new RuntimeException("Error loading menu texture.");

        //GLES20.glActiveTexture(GLES20.GL_TEXTURE1);

        GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, TextureHandle[1]);
        //GLES20.glTexImage2D(GLES20.GL_TEXTURE_2D, 0, GLES20.GL_RGBA, ui.menu_width, ui.menu_height, 0, GLES20.GL_RGBA, GLES20.GL_UNSIGNED_BYTE, ui.menu_bbuf);
        GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, ui.menu_bitmap, 0);
        GLES20.glTexParameteri ( GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_LINEAR );
        GLES20.glTexParameteri ( GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, GLES20.GL_LINEAR );
        GLES20.glTexParameteri ( GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S, GLES20.GL_CLAMP_TO_EDGE );
        GLES20.glTexParameteri ( GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T, GLES20.GL_CLAMP_TO_EDGE );

        //c64BufPositionParam=GLES20.glGetUniformLocation(screenProgram, "u_texture");
        menuTexCoordinate = GLES20.glGetAttribLocation(menuProgram, "a_texCoordinate");
        menuPositionParam = GLES20.glGetAttribLocation(menuProgram, "a_position");
        menuModelViewProjectionParam = GLES20.glGetUniformLocation(menuProgram, "u_sMVP");

        checkGLError("Menu program params");

        ByteBuffer bbMenuTexVertices=ByteBuffer.allocateDirect(WorldLayoutData.TEX_COORDS.length*4);
        bbMenuTexVertices.order(ByteOrder.nativeOrder());
        menuTexVertices=bbMenuTexVertices.asFloatBuffer();
        menuTexVertices.put(WorldLayoutData.TEX_COORDS);
        menuTexVertices.position(0);

        ByteBuffer bbMenuVertices=ByteBuffer.allocateDirect(WorldLayoutData.MENU_COORDS.length*4);
        bbMenuVertices.order(ByteOrder.nativeOrder());
        menuVertices=bbMenuVertices.asFloatBuffer();
        menuVertices.put(WorldLayoutData.MENU_COORDS);
        menuVertices.position(0);

        //Matrix.setIdentityM(modelFloor, 0);
        //Matrix.translateM(modelFloor, 0, 0, -floorDepth, 0); // Floor appears below user.

        // Avoid any delays during start-up due to decoding of sound files.
        //new Thread(
        //new Runnable() {
        //            @Override
        //            public void run() {
                        // Start spatial audio playback of OBJECT_SOUND_FILE at the model position. The
                        // returned sourceId handle is stored and allows for repositioning the sound object
                        // whenever the cube position changes.
                        //gvrAudioEngine.preloadSoundFile(OBJECT_SOUND_FILE);
                        //sourceId = gvrAudioEngine.createSoundObject(OBJECT_SOUND_FILE);
                        //gvrAudioEngine.setSoundObjectPosition(
                        //        sourceId, modelPosition[0], modelPosition[1], modelPosition[2]);
                        //gvrAudioEngine.playSound(sourceId, true /* looped playback */);
                        // Preload an unspatialized sound to be played on a successful trigger on the cube.
                        //gvrAudioEngine.preloadSoundFile(SUCCESS_SOUND_FILE);
        //            }
        //        })
        //        .start();

        updateMenuPosition();
        updateScreenPosition();

        checkGLError("onSurfaceCreated");
    }

    protected void updateMenuPosition() {
        Matrix.setIdentityM(modelMenu, 0);
        Matrix.translateM(modelMenu, 0, menuPosition[0], menuPosition[1], menuPosition[2]);
        //Matrix.rotateM(modelMenu, 0, 30.0f, -0.3f, 1.0f, 0.0f);
        //Matrix.rotateM(modelMenu, 0, -10.0f, 1.0f, 0.0f, 0.0f);
        Matrix.rotateM(modelMenu, 0, menuRotation[0], 1.0f, 0.0f, 0.0f);
        Matrix.rotateM(modelMenu, 0, menuRotation[1], 0.0f, 1.0f, 0.0f);
        Matrix.rotateM(modelMenu, 0, menuRotation[2], 0.0f, 0.0f, 1.0f);
    }

    protected void updateScreenPosition() {
        Matrix.setIdentityM(modelScreen, 0);
        Matrix.translateM(modelScreen, 0, screenPosition[0], screenPosition[1], screenPosition[2]);
        Matrix.rotateM(modelScreen, 0, screenRotation[0], 1.0f, 0.0f, 0.0f);
        Matrix.rotateM(modelScreen, 0, screenRotation[1], 0.0f, 1.0f, 0.0f);
        Matrix.rotateM(modelScreen, 0, screenRotation[2], 0.0f, 0.0f, 1.0f);

        // Update the sound location to match it with the new cube position.
        //if (sourceId != GvrAudioEngine.INVALID_ID) {
        //    gvrAudioEngine.setSoundObjectPosition(
        //            sourceId, screenPosition[0], screenPosition[1], screenPosition[2]);
        //}
        checkGLError("updateScreenPosition");
    }

    /**
     * Converts a raw text file, saved as a resource, into an OpenGL ES shader.
     *
     * @param type  The type of shader we will be creating.
     * @param resId The resource ID of the raw text file about to be turned into a shader.
     * @return The shader object handler.
     */
    private int loadGLShader(int type, int resId) {
        String code = readRawTextFile(resId);
        int shader = GLES20.glCreateShader(type);
        GLES20.glShaderSource(shader, code);
        GLES20.glCompileShader(shader);

        // Get the compilation status.
        final int[] compileStatus = new int[1];
        GLES20.glGetShaderiv(shader, GLES20.GL_COMPILE_STATUS, compileStatus, 0);

        // If the compilation failed, delete the shader.
        if (compileStatus[0] == 0) {
            Log.e(TAG, "Error compiling shader: " + GLES20.glGetShaderInfoLog(shader));
            GLES20.glDeleteShader(shader);
            shader = 0;
        }

        if (shader == 0) {
            throw new RuntimeException("Error creating shader.");
        }

        return shader;
    }

    /**
     * Converts a raw text file into a string.
     *
     * @param resId The resource ID of the raw text file about to be turned into a shader.
     * @return The context of the text file, or null in case of error.
     */
    private String readRawTextFile(int resId) {
        InputStream inputStream = getResources().openRawResource(resId);
        try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
            StringBuilder sb = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                sb.append(line).append("\n");
            }
            reader.close();
            return sb.toString();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Prepares OpenGL ES before we draw a frame.
     *
     * @param headTransform The head transformation in the new frame.
     */
    @Override
    public void onNewFrame(HeadTransform headTransform) {

        // Build the camera matrix and apply it to the ModelView.
        Matrix.setLookAtM(camera, 0, 0.0f, 0.0f, CAMERA_Z, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);

        headTransform.getHeadView(headView, 0);

        // Update the 3d audio engine with the most recent head rotation.
        headTransform.getQuaternion(headRotation, 0);
        //gvrAudioEngine.setHeadRotation(
        //        headRotation[0], headRotation[1], headRotation[2], headRotation[3]);
        // Regular update call to GVR audio engine.
        //gvrAudioEngine.update();

        headTransform.getForwardVector(forwardView, 0);

        setScreenLook();
        ui.lookJoystick();
        setMenuLook();
        ui.update_menu();

        checkGLError("onReadyToDraw");
    }

    //protected void setCubeRotation() {
    //    Matrix.rotateM(modelCube, 0, TIME_DELTA, 0.5f, 0.5f, 1.0f);
    //}

    /**
     * Draws a frame for an eye.
     *
     * @param eye The eye to render. Includes all required transformations.
     */
    @Override
    public void onDrawEye(Eye eye) {
        GLES20.glEnable(GLES20.GL_DEPTH_TEST);
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT | GLES20.GL_DEPTH_BUFFER_BIT);
        //checkGLError("colorParam");

        // Apply the eye transformation to the camera.
        Matrix.multiplyMM(view, 0, eye.getEyeView(), 0, camera, 0);

        // Set the position of the light
        //Matrix.multiplyMV(lightPosInEyeSpace, 0, view, 0, LIGHT_POS_IN_WORLD_SPACE, 0);

        // Build the ModelView and ModelViewProjection matrices
        // for calculating cube position and light.
        float[] perspective = eye.getPerspective(Z_NEAR, Z_FAR);
        //Matrix.multiplyMM(modelView, 0, view, 0, modelCube, 0);
        //Matrix.multiplyMM(modelViewProjection, 0, perspective, 0, modelView, 0);
        //drawCube();

        Matrix.multiplyMM(modelView, 0, view, 0, modelMenu, 0);
        Matrix.multiplyMM(modelViewProjection, 0, perspective, 0, modelView, 0);
        drawMenu();

        Matrix.multiplyMM(modelView, 0, view, 0, modelScreen, 0);
        Matrix.multiplyMM(modelViewProjection, 0, perspective, 0, modelView, 0);
        drawScreen();

        // Set modelView for the floor, so we draw floor in the correct location
        //Matrix.multiplyMM(modelView, 0, view, 0, modelFloor, 0);
        //Matrix.multiplyMM(modelViewProjection, 0, perspective, 0, modelView, 0);
        //drawFloor();
    }

    @Override
    public void onFinishFrame(Viewport viewport) {
    }

    public void drawMenu() {
        GLES20.glUseProgram(menuProgram);
        GLES20.glVertexAttribPointer(
                menuPositionParam, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, menuVertices);
        GLES20.glUniformMatrix4fv(menuModelViewProjectionParam, 1, false, modelViewProjection, 0);

        GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
        GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, TextureHandle[1]);
        if(ui.menu_updated) {
            GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, ui.menu_bitmap, 0);
            ui.menu_updated = false;
        }

        GLES20.glVertexAttribPointer(menuTexCoordinate, 2, GLES20.GL_FLOAT, false, 0, menuTexVertices);
        GLES20.glEnableVertexAttribArray(menuPositionParam);
        GLES20.glEnableVertexAttribArray(menuTexCoordinate);
        GLES20.glDrawArrays(GLES20.GL_TRIANGLES, 0, 6);
        GLES20.glDisableVertexAttribArray(menuPositionParam);
        GLES20.glDisableVertexAttribArray(menuTexCoordinate);
        checkGLError("Drawing menu");
    }

    public void drawScreen() {
        GLES20.glUseProgram(screenProgram);

        //GLES20.glUniform3fv(screenLightPosParam, 1, lightPosInEyeSpace, 0);

        // Set the Model in the shader, used to calculate lighting
        //GLES20.glUniformMatrix4fv(screenModelParam, 1, false, modelScreen, 0);

        // Set the ModelView in the shader, used to calculate lighting
        //GLES20.glUniformMatrix4fv(screenModelViewParam, 1, false, modelView, 0);

        // Set the position of the cube
        GLES20.glVertexAttribPointer(
                screenPositionParam, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, screenVertices);

        // Set the ModelViewProjection matrix in the shader.
        GLES20.glUniformMatrix4fv(screenModelViewProjectionParam, 1, false, modelViewProjection, 0);

        // Set the normal positions of the cube, again for shading
        //GLES20.glVertexAttribPointer(screenNormalParam, 3, GLES20.GL_FLOAT, false, 0, screenNormals);
        //GLES20.glVertexAttribPointer(screenColorParam, 4, GLES20.GL_FLOAT, false, 0, screenColors);

        GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
        //GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, textureHandle[0]);
        //GLES20.glUniform1i(c64BufPositionParam, 0);
        GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, TextureHandle[0]);
        if(vice_machine.videoUpdated()==1) {
            GLES20.glTexImage2D(GLES20.GL_TEXTURE_2D, 0, GLES20.GL_RGBA,
                    machine_video_width, machine_video_height,
                    0, GLES20.GL_RGBA, GLES20.GL_UNSIGNED_BYTE, machine_video_bbuf);
        }
        //GLES20.glEnable(GLES20.GL_TEXTURE_2D);
        GLES20.glVertexAttribPointer(screenTexCoordinate, 2, GLES20.GL_FLOAT, false, 0, screenTexVertices);

        // Enable vertex arrays
        GLES20.glEnableVertexAttribArray(screenPositionParam);
        //GLES20.glEnableVertexAttribArray(screenNormalParam);
        //GLES20.glEnableVertexAttribArray(screenColorParam);
        GLES20.glEnableVertexAttribArray(screenTexCoordinate);
        //GLES20.glEnableVertexAttribArray(c64BufPositionParam);

        GLES20.glDrawArrays(GLES20.GL_TRIANGLES, 0, 6);

        // Disable vertex arrays
        GLES20.glDisableVertexAttribArray(screenPositionParam);
        //GLES20.glDisableVertexAttribArray(screenNormalParam);
        //GLES20.glDisableVertexAttribArray(screenColorParam);
        GLES20.glDisableVertexAttribArray(screenTexCoordinate);
        //GLES20.glDisableVertexAttribArray(c64BufPositionParam);

        checkGLError("Drawing screen");
    }

    /**
     * Draw the cube.
     * <p>
     * <p>We've set all of our transformation matrices. Now we simply pass them into the shader.
     */
    //public void drawCube() {
    //    GLES20.glUseProgram(cubeProgram);

     //    GLES20.glUniform3fv(cubeLightPosParam, 1, lightPosInEyeSpace, 0);

        // Set the Model in the shader, used to calculate lighting
     //    GLES20.glUniformMatrix4fv(cubeModelParam, 1, false, modelCube, 0);

        // Set the ModelView in the shader, used to calculate lighting
     //   GLES20.glUniformMatrix4fv(cubeModelViewParam, 1, false, modelView, 0);

        // Set the position of the cube
     //   GLES20.glVertexAttribPointer(
     //           cubePositionParam, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, 0, cubeVertices);

        // Set the ModelViewProjection matrix in the shader.
     //   GLES20.glUniformMatrix4fv(cubeModelViewProjectionParam, 1, false, modelViewProjection, 0);

        // Set the normal positions of the cube, again for shading
      //  GLES20.glVertexAttribPointer(cubeNormalParam, 3, GLES20.GL_FLOAT, false, 0, cubeNormals);
      //  GLES20.glVertexAttribPointer(cubeColorParam, 4, GLES20.GL_FLOAT, false, 0,
      //          isLookingAtObject() ? cubeFoundColors : cubeColors);

        // Enable vertex arrays
      //  GLES20.glEnableVertexAttribArray(cubePositionParam);
      //  GLES20.glEnableVertexAttribArray(cubeNormalParam);
      //  GLES20.glEnableVertexAttribArray(cubeColorParam);

      //  GLES20.glDrawArrays(GLES20.GL_TRIANGLES, 0, 36);

        // Disable vertex arrays
      //  GLES20.glDisableVertexAttribArray(cubePositionParam);
      //  GLES20.glDisableVertexAttribArray(cubeNormalParam);
      //  GLES20.glDisableVertexAttribArray(cubeColorParam);

       // checkGLError("Drawing cube");
    //}

    private void recenter() {
        float[] rotationMatrix = new float[16];
        float[] posVec = new float[4];

        Matrix.multiplyMM(modelView, 0, headView, 0, modelScreen, 0);
        Matrix.multiplyMV(tempPosition, 0, modelView, 0, POS_MATRIX_MULTIPLY_VEC, 0);
        float angleXZ = (float) Math.atan2(tempPosition[0], -tempPosition[2]);
        angleXZ=(float)Math.toDegrees(angleXZ);
        Matrix.setRotateM(rotationMatrix, 0, angleXZ, 0f, 1f, 0f);
        Matrix.multiplyMV(posVec, 0, rotationMatrix, 0, modelScreen, 12);
        screenRotation[1]+=angleXZ;
        screenPosition[0] = posVec[0];
        screenPosition[2] = posVec[2];
        updateScreenPosition();

        Matrix.multiplyMM(modelView, 0, headView, 0, modelMenu, 0);
        Matrix.multiplyMV(tempPosition, 0, modelView, 0, POS_MATRIX_MULTIPLY_VEC, 0);
        angleXZ = (float) Math.atan2(tempPosition[0], -tempPosition[2]);
        angleXZ=(float)Math.toDegrees(angleXZ);
        Matrix.setRotateM(rotationMatrix, 0, angleXZ, 0f, 1f, 0f);
        Matrix.multiplyMV(posVec, 0, rotationMatrix, 0, modelMenu, 12);
        menuRotation[1]+=angleXZ;
        menuPosition[0] = posVec[0];
        menuPosition[2] = posVec[2];
        updateMenuPosition();
    }

    /**
     * Called when the Cardboard trigger is pulled.
     */
    @Override
    public void onCardboardTrigger() {
        //Log.i(TAG, "onCardboardTrigger");
        vibrator.vibrate(50);
        ui.cardboard_trigger();
        if(!ui.cursor_on_menu() && !ui.cursor_on_screen()) recenter();
    }

    /**
     * Find a new random position for the object.
     * <p>
     * <p>We'll rotate it around the Y-axis so it's out of sight, and then up or down by a little bit.
     */
    //protected void hideObject() {
    //    float[] rotationMatrix = new float[16];
    //    float[] posVec = new float[4];

        // First rotate in XZ plane, between 90 and 270 deg away, and scale so that we vary
        // the object's distance from the user.
     //   float angleXZ = (float) Math.random() * 180 + 90;
     //   Matrix.setRotateM(rotationMatrix, 0, angleXZ, 0f, 1f, 0f);
     //   float oldObjectDistance = objectDistance;
     //   objectDistance =
     //           (float) Math.random() * (MAX_MODEL_DISTANCE - MIN_MODEL_DISTANCE) + MIN_MODEL_DISTANCE;
     //   float objectScalingFactor = objectDistance / oldObjectDistance;
     //   Matrix.scaleM(rotationMatrix, 0, objectScalingFactor, objectScalingFactor, objectScalingFactor);
     //   Matrix.multiplyMV(posVec, 0, rotationMatrix, 0, modelCube, 12);

     //   float angleY = (float) Math.random() * 80 - 40; // Angle in Y plane, between -40 and 40.
     //   angleY = (float) Math.toRadians(angleY);
     //   float newY = (float) Math.tan(angleY) * objectDistance;

     //   modelPosition[0] = posVec[0];
     //   modelPosition[1] = newY;
     //   modelPosition[2] = posVec[2];

     //   updateModelPosition();
    //}

    private void setScreenLook() {
        Matrix.multiplyMM(modelView, 0, headView, 0, modelScreen, 0);
        Matrix.multiplyMV(tempPosition, 0, modelView, 0, POS_MATRIX_MULTIPLY_VEC, 0);
        ui.screen_pitch = (float) Math.atan2(tempPosition[1], -tempPosition[2]);
        ui.screen_yaw = (float) Math.atan2(tempPosition[0], -tempPosition[2]);
    }

    private void setMenuLook() {
        Matrix.multiplyMM(modelView, 0, headView, 0, modelMenu, 0);
        Matrix.multiplyMV(tempPosition, 0, modelView, 0, POS_MATRIX_MULTIPLY_VEC, 0);
        ui.menu_pitch = (float) Math.atan2(tempPosition[1], -tempPosition[2]);
        ui.menu_yaw = (float) Math.atan2(tempPosition[0], -tempPosition[2]);
    }

    private void get_android_permissions() {
        int permissionCheck = ContextCompat.checkSelfPermission(this.getApplicationContext(),
                permission.READ_EXTERNAL_STORAGE);
        if(permissionCheck==PackageManager.PERMISSION_GRANTED) return;
        Log.i(TAG, "get_android_permission: permissions required");

        if (ActivityCompat.shouldShowRequestPermissionRationale(this,
                permission.READ_EXTERNAL_STORAGE)) {
        } else {
            ActivityCompat.requestPermissions(this,
                    new String[]{permission.READ_EXTERNAL_STORAGE}, 0);
        }
    }

    public boolean dispatchKeyEvent(KeyEvent event) {
        ui.keyEvent(event);
        return true;
    }
}
