/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

package org.gp2x.cvice;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.Log;
import android.view.KeyEvent;

import java.io.File;
import java.util.Arrays;


public class UserInterface {
    private static final String TAG = "VRVice UI";

    VICEMachine vice_machine;

    public boolean menu_updated;
    private boolean menu_updating;
    public float menu_pitch;
    public float menu_yaw;
    private int menu_cursor_x;
    private int menu_cursor_y;

    public float screen_pitch;
    public float screen_yaw;

    private int menu_width=500;
    private int menu_height=500;
    private static final int menu_text_start=100;
    private static final int menu_text_height=50;

    private int look_joystick;
    private boolean look_joy_autofire;
    private int look_joy_fire_state;
    private int gamepad_port=2;
    private int gamepad_up, gamepad_down,
            gamepad_left, gamepad_right, gamepad_fire;

    private File[] directory_list;
    private String current_directory="/sdcard";
    private final int directory_list_menu_size=8;
    private int directory_list_start;

    private enum menus {
        MAIN, LOAD
    }
    private menus current_menu;

    private enum menu_options {
        LOAD, GAMEPAD_PORT, LOOK_JOY, LOOK_JOY_AUTO, RESET
    }
    private class menu_item {
        public menu_options option;
        public String text;
        menu_item(menu_options x, String y) {
            option=x;
            text=y;
        }
    }
    private menu_item[] main_menu = new menu_item[] {
            new menu_item(menu_options.LOAD,            "Load"),
            new menu_item(menu_options.GAMEPAD_PORT,    "Gamepad"),
            new menu_item(menu_options.LOOK_JOY,        "Look-joystick"),
            new menu_item(menu_options.LOOK_JOY_AUTO,   "Look-joystick autofire"),
            new menu_item(menu_options.RESET,           "Reset"),
    };

    public Bitmap menu_bitmap;
    private Canvas menu_canvas;
    private Paint textPaint;

    UserInterface(VICEMachine m) {
        vice_machine=m;

        menu_bitmap=Bitmap.createBitmap(menu_width, menu_height, Bitmap.Config.ARGB_4444);
        menu_canvas=new Canvas(menu_bitmap);
        textPaint=new Paint();
        textPaint.setTextSize(32);
        textPaint.setAntiAlias(true);

        current_menu=menus.MAIN;
        menu_updated=true;
        menu_updating=true;
    }

    private void draw_load_menu() {
        for(int i=0; i<directory_list_menu_size; i++) {
            if (i == get_cursor_menu_pos()) {
                textPaint.setARGB(0xff, 0xff, 0xff, 0x00);
            } else {
                textPaint.setARGB(0xff, 0xff, 0xff, 0xff);
            }
            if(i==0) {
                menu_canvas.drawText("Back", 10, menu_text_start + (i * menu_text_height), textPaint);
                continue;
            } else if(i==1) {
                menu_canvas.drawText("  ^^^", 10, menu_text_start + (i * menu_text_height), textPaint);
                continue;
            } else if(i==2) {
                menu_canvas.drawText("..", 10, menu_text_start + (i * menu_text_height), textPaint);
                continue;
            } else if(i==directory_list_menu_size-1) {
                menu_canvas.drawText("  vvv", 10, menu_text_start + (i * menu_text_height), textPaint);
                continue;
            }
            int dir_entry=i-3+directory_list_start;
            if(directory_list!=null && dir_entry<directory_list.length) {
                if(directory_list[dir_entry].isDirectory()) {
                    menu_canvas.drawText("<dir> "+directory_list[dir_entry].getName(), 10, menu_text_start + (i * menu_text_height), textPaint);
                } else {
                    menu_canvas.drawText(directory_list[dir_entry].getName(), 10, menu_text_start + (i * menu_text_height), textPaint);
                }
            }
        }
    }

    public void lookJoystick() {
        if(look_joystick==0) return;
        int port=look_joystick-1;

        int up=0;
        int down=0;
        int left=0;
        int right=0;
        if(screen_pitch<-0.1) up=1;
        if(screen_pitch>0.1) down=1;
        if(screen_yaw>0.1) left=1;
        if(screen_yaw<-0.1) right=1;

        if(look_joy_autofire) {
            if(look_joy_fire_state==1) look_joy_fire_state=0;
            else look_joy_fire_state=1;
        } else {
            look_joy_fire_state=0;
        }
        //Log.i(TAG, "setJoystick: "+port+up+down+left+right+look_joy_fire_state);
        vice_machine.setJoystick(port, up, down, left, right, look_joy_fire_state);
    }

    public void update_menu() {
        if(menu_updating) {
            menu_bitmap.eraseColor(Color.GRAY);
            textPaint.setARGB(0xff, 0xff, 0xff, 0xff);
            menu_canvas.drawText("VR VICE", 10, 50, textPaint);
            menu_updated=true;
            menu_updating=false;

            if(current_menu==menus.MAIN) {
                for (int i = 0; i < main_menu.length; i++) {
                    if (i == get_cursor_menu_pos()) {
                        textPaint.setARGB(0xff, 0xff, 0xff, 0x00);
                    } else {
                        textPaint.setARGB(0xff, 0xff, 0xff, 0xff);
                    }
                    menu_canvas.drawText(main_menu[i].text, 10, menu_text_start + (i*menu_text_height), textPaint);
                    if(main_menu[i].option==menu_options.GAMEPAD_PORT) {
                        switch(gamepad_port) {
                            case 1:
                                menu_canvas.drawText("Port 1", 400, menu_text_start + ((i) * menu_text_height), textPaint);
                                break;
                            case 2:
                                menu_canvas.drawText("Port 2", 400, menu_text_start + ((i) * menu_text_height), textPaint);
                                break;
                        }

                    }
                    if (main_menu[i].option == menu_options.LOOK_JOY) {
                        switch (look_joystick) {
                            case 0:
                                menu_canvas.drawText("Off", 400, menu_text_start + ((i) * menu_text_height), textPaint);
                                break;
                            case 1:
                                menu_canvas.drawText("Port 1", 400, menu_text_start + ((i) * menu_text_height), textPaint);
                                break;
                            case 2:
                                menu_canvas.drawText("Port 2", 400, menu_text_start + ((i) * menu_text_height), textPaint);
                                break;
                        }
                    } else if(main_menu[i].option == menu_options.LOOK_JOY_AUTO) {
                        if(look_joy_autofire) {
                            menu_canvas.drawText("On", 400, menu_text_start + ((i) * menu_text_height), textPaint);
                        } else {
                            menu_canvas.drawText("Off", 400, menu_text_start + ((i) * menu_text_height), textPaint);
                        }
                    }
                }
            } else if (current_menu == menus.LOAD) draw_load_menu();
        }
        if(cursor_on_menu()) {
            set_menu_cursor();
            textPaint.setARGB(0xff, 0xff, 0xff, 0x00);
            menu_canvas.drawText("*", menu_cursor_x, menu_cursor_y, textPaint);
            menu_updated=true;
            menu_updating=true;
        }
    }

    private void read_directory(String dir) {
        File f = new File(dir);
        directory_list = f.listFiles();
        Arrays.sort(directory_list);
        if(directory_list!=null) {
            current_directory = dir;
            //for (File inFile : directory_list) {
            //    Log.i(TAG, "read_directory: " + inFile.getName());
            //}
        }
        directory_list_start=0;
    }

    public void cardboard_trigger() {
        int i=get_cursor_menu_pos();
        if(i==-1) return;
        if(current_menu==menus.MAIN) {
            if(i>=main_menu.length) return;
            switch (main_menu[i].option) {
                case LOAD:
                    read_directory(current_directory);
                    current_menu = menus.LOAD;
                    break;
                case GAMEPAD_PORT:
                    if(gamepad_port==1) gamepad_port=2;
                    else gamepad_port=1;
                    break;
                case LOOK_JOY:
                    look_joystick++;
                    if (look_joystick > 2) look_joystick = 0;
                    break;
                case LOOK_JOY_AUTO:
                    look_joy_autofire=!look_joy_autofire;
                    break;
                case RESET:
                    vice_machine.viceUIReset();
                    break;
            }
        } else if(current_menu==menus.LOAD) {
            Log.i(TAG, "trigger: "+i);
            switch(i) {
                case 0:
                    current_menu=menus.MAIN;
                    break;
                case 1:
                    directory_list_start-=(directory_list_menu_size-4);
                    if(directory_list_start<0) directory_list_start=0;
                    break;
                case 2:
                    try {
                        File f=new File(current_directory);
                        read_directory(f.getParent());
                    } catch(Exception e) {
                        read_directory("/");
                    }
                    break;
                case directory_list_menu_size-1:
                    directory_list_start+=(directory_list_menu_size-4);
                    break;
                default:
                    int dir_entry=i-3+directory_list_start;
                    if(directory_list==null || dir_entry>=directory_list.length) break;
                    if(directory_list[dir_entry].isDirectory()) {
                        try {
                            read_directory(directory_list[dir_entry].getCanonicalPath());
                        } catch(Exception e) {
                            read_directory("/");
                        }
                    } else {
                        vice_machine.loadFile(current_directory+File.separatorChar+directory_list[dir_entry].getName());
                    }
                    break;
            }

        }
    }

    public boolean cursor_on_menu() {
        if(menu_pitch<-0.1 || menu_pitch>0.1) return false;
        if(menu_yaw<-0.1 || menu_yaw>0.1) return false;
        return true;
    }

    public boolean cursor_on_screen() {
        if(screen_pitch<-0.2 || screen_pitch>0.2) return false;
        if(screen_yaw<-0.3 || screen_yaw>0.3) return false;
        return true;
    }

    private void set_menu_cursor() {
        menu_cursor_x=(int)(((-menu_yaw+0.1)*5.0)*(float)menu_width);
        menu_cursor_y=(int)(((menu_pitch+0.1)*5.0)*(float)menu_height);
        //Log.i(TAG, "cursor_x: "+menu_cursor_x+", cursor_y: "+menu_cursor_y);
    }

    private int get_cursor_menu_pos() {
        if(!cursor_on_menu()) return -1;
        set_menu_cursor();
        int i=menu_cursor_y-menu_text_start;
        return i/menu_text_height;
    }

    public void keyEvent(KeyEvent event) {
        //Log.i(TAG, "dispatchKeyEvent: "+event.toString());

        boolean update_joystick=false;
        int action;
        if(event.getAction()==KeyEvent.ACTION_DOWN) action=1;
        else action=0;

        switch(event.getKeyCode()) {
            case KeyEvent.KEYCODE_DPAD_UP:
                gamepad_up=action;
                update_joystick=true;
                break;
            case KeyEvent.KEYCODE_DPAD_DOWN:
                gamepad_down=action;
                update_joystick=true;
                break;
            case KeyEvent.KEYCODE_DPAD_LEFT:
                gamepad_left=action;
                update_joystick=true;
                break;
            case KeyEvent.KEYCODE_DPAD_RIGHT:
                gamepad_right=action;
                update_joystick=true;
                break;
            case KeyEvent.KEYCODE_BUTTON_A:
            case KeyEvent.KEYCODE_BUTTON_X:
                gamepad_fire=action;
                update_joystick=true;
                break;
        }
        if(update_joystick) {
            //Log.i(TAG, "dispatchKeyEvent: "+event.toString());
            vice_machine.setJoystick(gamepad_port-1,
                    gamepad_up, gamepad_down, gamepad_left, gamepad_right, gamepad_fire);
        }

        vice_machine.keyPress(action, event.getKeyCode());
    }
}
