/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

package org.gp2x.cvice;

import android.content.Context;

public class VICEMachine extends Thread {

    public enum vice_machine {
        X64, X64SC, XPLUS4, XVIC
    }
    public vice_machine machine;

    public native void keyPress(int down, int key);
    public native void loadFile(String file);
    public native void setJoystick(int port, int up, int down, int left, int right, int fire);
    public native void vicePause();
    public native void viceResume();
    public native void viceExit();
    public native void viceUIReset();
    public native int videoUpdated();

    private static native int cx64Init(String datadir, byte[] vbuf);
    private static native int cx64scInit(String datadir, byte[] vbuf);
    private static native int cxplus4Init(String datadir, byte[] vbuf);
    private static native int cxvicInit(String datadir, byte[] vbuf);

    private Context ctx;
    private byte[] video_buf;

    VICEMachine(vice_machine m, Context c, byte[] buf) {
        machine=m;
        switch(machine) {
            case X64:
                System.loadLibrary("x64");
                break;
            case X64SC:
                System.loadLibrary("x64sc");
                break;
            case XPLUS4:
                System.loadLibrary("xplus4");
                break;
            case XVIC:
                System.loadLibrary("xvic");
                break;
        }

        this.ctx=c;
        this.video_buf=buf;
    }

    public void run() {
        FileManagement fm=new FileManagement();
        fm.SyncAssets(ctx, "");

        switch(machine) {
            case X64:
                cx64Init(fm.getDirPath(ctx), video_buf);
                break;
            case X64SC:
                cx64scInit(fm.getDirPath(ctx), video_buf);
                break;
            case XPLUS4:
                cxplus4Init(fm.getDirPath(ctx), video_buf);
                break;
            case XVIC:
                cxvicInit(fm.getDirPath(ctx), video_buf);
                break;
        }
    }
}
